@@ -6,16 +6,17 @@
 ### Author: Sebastian Meyer
 ################################################################################
 
-# CAVE: the type of contrasts for factor variables has to be set through options("contrasts")
-
-### TODO: if epidemic-only process (!hash), we actually don't need stgrid
-###       the function is not yet arranged for endemic-only or epidemic-only settings
+### CAVE:
+### - the type of contrasts for factor variables has to be set through options("contrasts")
+### - if epidemic-only process (!hash), we actually don't need stgrid, but we
+###   want to have valid epidataCS at the end, which requires stgrid
 
 simEpidataCS <- function (endemic, epidemic, siaf, tiaf, qmatrix, rmarks,
     events, stgrid, tiles, beta0, beta, gamma, siafpars, tiafpars,
-    t0 = stgrid$start[1], T = tail(stgrid$stop,1), nEvents = 1e5, nCub,
+    t0 = stgrid$start[1], T = tail(stgrid$stop,1), nEvents = 1e5,
+    nCub, nCub.adaptive = FALSE,
     W = NULL, trace = 5, nCircle2Poly = 32, gmax = NULL, .allocate = 500,
-    .skipChecks = FALSE, .onlyEvents = FALSE, ...)
+    .skipChecks = FALSE, .onlyEvents = FALSE)
 {
     ptm <- proc.time()[[3]]
     cl <- match.call()
@@ -116,8 +117,10 @@ simEpidataCS <- function (endemic, epidemic, siaf, tiaf, qmatrix, rmarks,
     )
 
 
-    ### Check class of W, and class, proj4string and names of tiles
+    ### Check class, proj4string and names of tiles and W
 
+    stopifnot(inherits(tiles, "SpatialPolygons"),
+              (tileLevels <- levels(stgrid$tile)) %in% row.names(tiles))
     if (is.null(W)) {
     	if (require("maptools")) {
             cat("Building W as the union of 'tiles' ...\n")
@@ -127,10 +130,10 @@ simEpidataCS <- function (endemic, epidemic, siaf, tiaf, qmatrix, rmarks,
         } else {
             stop("automatic generation of 'W' from 'tiles' requires package \"maptools\"")
         }
+    } else {
+        stopifnot(inherits(W, "SpatialPolygons"),
+                  identical(proj4string(tiles), proj4string(W)))
     }
-    stopifnot(inherits(W, "SpatialPolygons"), inherits(tiles, "SpatialPolygons"),
-              proj4string(tiles) == proj4string(W),
-              (tileLevels <- levels(stgrid$tile)) %in% row.names(tiles))
 
     # Transform W into a gpc.poly
     Wgpc <- as(W, "gpc.poly")
@@ -138,27 +141,23 @@ simEpidataCS <- function (endemic, epidemic, siaf, tiaf, qmatrix, rmarks,
 
     ### Check mark-generating function
 
-    epscols <- c("eps.t", "eps.s")
-    unpredMarks <- setdiff(unique(c(if (hase) epscols, all.vars(epidemic))), "type")
-    # <- eps.t and eps.s are also taken to be unpredictable marks
-    rmarks <- if (missing(rmarks) || is.null(rmarks)) NULL else match.fun(rmarks)
-    hasMarks <- !is.null(rmarks)
-    if (hasMarks) {
-        sampleCoordinate <- coordinates(spsample(tiles, n=1L, type="random"))
-        sampleMarks <- rmarks(t0, sampleCoordinate)
-        # should be a one-row data.frame
-        if (!is.data.frame(sampleMarks) || nrow(sampleMarks) != 1L) {
-            stop("'rmarks' must return a one-row data.frame of marks")
-        }
-        if (!all(sapply(sampleMarks, function(x) inherits(x, c("integer","numeric","factor"), which=FALSE)))) {
-            stop("'rmarks' must return \"numeric\", \"integer\" or \"factor\" variables only")
-        }
-        markNames <- names(sampleMarks)
-        if (.idx <- match(FALSE, unpredMarks %in% markNames, nomatch=0L)) {
-            stop("the unpredictable mark '", unpredMarks[.idx], "' is not returned by 'rmarks'")
-        }
-    } else if (length(unpredMarks) > 0L) {
-        stop("missing mark-generating function 'rmarks' for the 'epidemic' component")
+    # eps.t and eps.s are also taken to be unpredictable marks (generated by rmarks)
+    unpredMarks <- unique(c("eps.t", "eps.s", if (hase) {
+        setdiff(all.vars(epidemic), "type")
+    }))
+    rmarks <- match.fun(rmarks)
+    sampleCoordinate <- coordinates(spsample(tiles, n=1L, type="random"))
+    sampleMarks <- rmarks(t0, sampleCoordinate)
+    # should be a one-row data.frame
+    if (!is.data.frame(sampleMarks) || nrow(sampleMarks) != 1L) {
+        stop("'rmarks' must return a one-row data.frame of marks")
+    }
+    if (!all(sapply(sampleMarks, function(x) inherits(x, c("integer","numeric","factor"), which=FALSE)))) {
+        stop("'rmarks' must return \"numeric\", \"integer\" or \"factor\" variables only")
+    }
+    markNames <- names(sampleMarks)
+    if (.idx <- match(FALSE, unpredMarks %in% markNames, nomatch=0L)) {
+        stop("the unpredictable mark '", unpredMarks[.idx], "' is not returned by 'rmarks'")
     }
 
 
@@ -173,9 +172,7 @@ simEpidataCS <- function (endemic, epidemic, siaf, tiaf, qmatrix, rmarks,
         type = factor(character(0L), levels=typeNames),
         check.rows = FALSE, check.names = FALSE
     )
-    if (hasMarks) {
-        eventData <- cbind(eventData, sampleMarks[0L,])
-    }
+    eventData <- cbind(eventData, sampleMarks[0L,])
     Nout <- 0L
     # do we have a prehistory in 'events'
     if (!missing(events) && !is.null(events)) {
@@ -185,7 +182,6 @@ simEpidataCS <- function (endemic, epidemic, siaf, tiaf, qmatrix, rmarks,
             cat("Checking 'events':\n")
             events <- checkEvents(events, dropTypes = FALSE)
             # epscols are obligatory in 'checkEvents', which is also appropriate here
-            # because in the case of endemic-only processes we would not need a prehistory at all
         }
         # select prehistory of events which are still infective
         .stillInfective <- with(events@data, time <= t0 & time + eps.t > t0)
@@ -255,9 +251,14 @@ simEpidataCS <- function (endemic, epidemic, siaf, tiaf, qmatrix, rmarks,
     # check if we have an endemic component at all
     hasOffset <- !is.null(attr(endemic, "offset"))
     hash <- (nbeta0 + p + hasOffset) > 0L
-    if (!hash && !hase) {
-        stop("nothing to do: neither endemic nor epidemic parameters were specified")
-        # actually, the process might be endemic offset-only, which I don't care about ATM
+    if (!hash) {
+        if (!hase) {
+            stop("nothing to do: neither endemic nor epidemic parameters were specified")
+            # actually, the process might be endemic offset-only, which I don't care about ATM
+        }
+        if (Nout == 0L) {
+            stop("missing 'events' pre-history (no endemic component)")
+        }
     }
 
     # remove (1|type) specification
@@ -265,8 +266,8 @@ simEpidataCS <- function (endemic, epidemic, siaf, tiaf, qmatrix, rmarks,
         "1 | type" %in% attr(endemic, "term.labels") || nbeta0 > 1
     if (typeSpecificEndemicIntercept) {
         endemic <- update(endemic, ~ . - (1|type)) # this drops the terms attributes
-        endemic <- terms(endemic, data = data$stgrid, keep.order = TRUE)
-        if (nbeta0 < 1L) {
+        endemic <- terms(endemic, data = stgrid, keep.order = TRUE)
+        if (nbeta0 <= 1L) {
             stop("for type-specific endemic intercepts, 'beta0' must be longer than 1")
         }
     }
@@ -280,13 +281,14 @@ simEpidataCS <- function (endemic, epidemic, siaf, tiaf, qmatrix, rmarks,
     # helper function taking eventData (with time and tile columns) and returning the endemic model.matrix
     buildmmh <- function (eventData)
     {
-        # if 'pi' appears in 'endemic' we don't care, and if a true covariate is missing, model.frame will throw an error
+        # if 'pi' appears in 'endemic' we don't care, and if a true covariate is
+        # missing, model.frame will throw an error
         # attach endemic covariates from 'stgrid' to events
         gridcellsOfEvents <- integer(nrow(eventData))
         for (i in seq_along(gridcellsOfEvents)) {
             gridcellsOfEvents[i] <- gridcellOfEvent(eventData[i,"time"], eventData[i,"tile"], stgrid)
         }
-        eventData <- cbind(eventData, stgrid[gridcellsOfEvents, stgridCopyCols])
+        eventData <- cbind(eventData, stgrid[gridcellsOfEvents, stgridCopyCols, drop=FALSE])
         # construct model matrix
         mfhEvents <- model.frame(endemic, data = eventData, na.action = na.fail, drop.unused.levels = FALSE)
         mmhEvents <- model.matrix(endemic, mfhEvents)
@@ -295,8 +297,8 @@ simEpidataCS <- function (endemic, epidemic, siaf, tiaf, qmatrix, rmarks,
         structure(mmhEvents, offset = model.offset(mfhEvents))
     }
 
-    # actually, we don't need the endemic model matrix for the events at all
-    # this is just to test consistence with 'beta'
+    # actually, we don't need the endemic model matrix for the pre-history events at all
+    # this is just to test consistence with 'beta' and for the names of 'beta'
     mmh <- buildmmh(eventData[0L,])
     if (ncol(mmh) != p) {
         stop("length of 'beta' (", p, ") does not match the 'endemic' specification (", ncol(mmh), ")")
@@ -324,33 +326,41 @@ simEpidataCS <- function (endemic, epidemic, siaf, tiaf, qmatrix, rmarks,
 
         ## Check interaction functions
         siaf <- do.call(".parseiaf", args = alist(siaf))
-        tiaf <- do.call(".parseiaf", args = alist(tiaf))
-
-        ## Spatially constant interaction siaf(s) = 1
-        constantsiaf <- is.null(siaf) || attr(siaf, "constant")
-        if (constantsiaf) siaf <- siaf.constant()
+        constantsiaf <- attr(siaf, "constant")
         if (siaf$npars != nsiafpars) {
             stop("length of 'siafpars' (", nsiafpars,
                  ") does not match the 'siaf' specification (", siaf$npars, ")")
         }
-
-        ## Temporally constant interaction tiaf(t) = 1
-        constanttiaf <- is.null(tiaf) || attr(tiaf, "constant")
-        if (constanttiaf) {
-            tiaf <- tiaf.constant()
-            gmax <- 1L
-        }
+        
+        tiaf <- do.call(".parseiaf", args = alist(tiaf))
+        constanttiaf <- attr(tiaf, "constant")
+        if (constanttiaf) gmax <- 1L
         if (tiaf$npars != ntiafpars) {
             stop("length of 'tiafpars' (", ntiafpars,
                  ") does not match the 'tiaf' specification (", tiaf$npars, ")")
         }
 
+        ## Define function that integrates the two-dimensional 'siaf' function
+        ## over the influence regions of the events
+        if (!constantsiaf && !is.null(siaf$Fcircle) && !is.null(siaf$effRange))
+        {
+            ## pre-compute effective range of the 'siaf' (used by .siafInt)
+            effRangeTypes <- rep(siaf$effRange(siafpars), length.out = nTypes)
+        }
+        .siafInt <- .siafIntFUN(siaf = siaf, nCub.adaptive = nCub.adaptive,
+                                noCircularIR = FALSE) # not certain beforehand
+        ## CAVE: nCub or nCub.adaptive might have been fixed by the above call
+
         ## Check nCub
         if (!constantsiaf) {
-            nCub <- as.integer(nCub)
-            if (any(nCub <= 0L)) {
+            stopifnot(is.vector(nCub, mode="numeric"))
+            if (any(is.na(nCub) | nCub <= 0L)) {
                 stop("'nCub' must be positive")
             }
+            if (isTRUE(cl[["nCub.adaptive"]]) && !nCub.adaptive) {
+                message("'nCub.adaptive' only works in conjunction with ",
+                        "specified 'siaf$effRange()'")
+            }
         }
 
         ## Check gmax
@@ -362,10 +372,12 @@ simEpidataCS <- function (endemic, epidemic, siaf, tiaf, qmatrix, rmarks,
         }
 
     } else {
-        if ((!missing(siaf) && !is.null(siaf)) || (!missing(tiaf) && !is.null(tiaf))) {
-            warning("'siaf' and 'tiaf' can only be modelled in conjunction with an 'epidemic' process")
-        }
+        if (!missing(siaf) && !is.null(siaf))
+            warning("'siaf' can only be modelled in conjunction with an 'epidemic' process")
+        if (!missing(tiaf) && !is.null(tiaf))
+            warning("'tiaf' can only be modelled in conjunction with an 'epidemic' process")
         siaf <- tiaf <- NULL
+        nCub <- nCub.adaptive <- NULL
     }
 
 
@@ -373,7 +385,8 @@ simEpidataCS <- function (endemic, epidemic, siaf, tiaf, qmatrix, rmarks,
 
     txtPrehistory <- if (Nout == 0L) "no prehistory" else paste(Nout,
         ngettext(Nout, "event", "events"), "in the prehistory")
-    cat("\nSimulating a", if (hasMarks) "marked", "spatio-temporal point pattern with",
+    cat("\nSimulating a", if (length(unpredMarks) > 2L) "marked",
+        "spatio-temporal point pattern with",
         "\n\t-", nTypes, ngettext(nTypes, "event type", "event types"),
         "\n\t-", txtPrehistory)
 
@@ -396,82 +409,61 @@ simEpidataCS <- function (endemic, epidemic, siaf, tiaf, qmatrix, rmarks,
     ##########################################
 
 
-    ### calculate integral of endemic component over W (= union of tiles) and types for all time blocks in stgrid
+    ### calculate integral of endemic component over W (= union of tiles)
+    ### and over types for all time blocks in stgrid
 
-    dsexpeta <- local({
-        eta <- drop(mmhGrid %*% beta)
-        if (!is.null(offsetGrid)) eta <- offsetGrid + eta
-        ds * exp(unname(eta))
-    })
-    fact <- if (nbeta0 > 1L) sum(exp(beta0)) else if (nbeta0 == 1L) nTypes*exp(unname(beta0)) else nTypes
-    hIntWK <- fact * c(tapply(dsexpeta, gridBlocks, sum))
+    hIntWK <- if (hash) {
+        dsexpeta <- local({
+            eta <- drop(mmhGrid %*% beta)
+            if (!is.null(offsetGrid)) eta <- offsetGrid + eta
+            ds * exp(unname(eta))
+        })
+        fact <- if (nbeta0 > 1L) sum(exp(beta0)) else if (nbeta0 == 1L) nTypes*exp(unname(beta0)) else nTypes
+        fact * c(tapply(dsexpeta, gridBlocks, sum))
+    } else structure(numeric(nrow(blockstarts)), names=blockstarts[,1]) # zeroes
     #<- is a named vector with names referencing BLOCK in stgrid
 
 
-    ### calculate terms appearing in the epidemic component (the lambdag function uses eTerms)
-
-    # compute computationally effective range of the 'siaf' function
-    if (!constantsiaf && !is.null(siaf$Fcircle)) {
-        effRangeTypes <- rep(siaf$effRange(siafpars), length.out = nTypes)
-    }
-
-    # helper function evaluating the epidemic terms of the ground intensity for a specific set of events
+    ### helper function evaluating the epidemic terms of the ground intensity
+    ### for a specific set of events (the lambdag function uses eTerms)
+    
     eTermsCalc <- function (eventData, eventCoords)
     {
-        # epidemic model matrix (will be multiplied with gamma)
-        mme <- buildmme(eventData)
+        # extract some marks from the eventData
+        eventTypes <- as.integer(eventData$type)
+        eps.s <- eventData$eps.s
         # distance to the border (required for siafInt below, and for epidataCS)
-        .bdist <- bdist(eventCoords, Wgpc)
+        bdist <- bdist(eventCoords, Wgpc)
         # spatial influence regions of the events
-        .influenceRegion <- if (nrow(eventCoords) > 0L) .influenceRegions(
+        influenceRegion <- if (nrow(eventCoords) > 0L) .influenceRegions(
             events = SpatialPointsDataFrame(
                 coords = eventCoords,
-                data = data.frame(eps.s = eventData$eps.s, .bdist = .bdist),
+                data = data.frame(eps.s = eps.s, .bdist = bdist),
                 match.ID = FALSE
             ),
             Wgpc = Wgpc,
             npoly = nCircle2Poly
         ) else list()
+        # epidemic terms
+        if (!hase) {
+            return(list(matrix(NA_real_, length(influenceRegion), 3L),
+                        bdist, influenceRegion))
+        }
+        # epidemic model matrix (will be multiplied with gamma)
+        mme <- buildmme(eventData)
         # integrate the two-dimensional 'siaf' function over the influence region
-        siafInt <- if (length(.influenceRegion) == 0L) numeric(0L) else if (constantsiaf) {
-                sapply(.influenceRegion, attr, "area")
-            } else if (is.null(siaf$Fcircle)) { # if siaf$Fcircle is not available
-                sapply(seq_along(.influenceRegion), function (i) {
-                    polyCub.midpoint(.influenceRegion[[i]], siaf$f, siafpars, as.integer(eventData[i,"type"]), eps = nCub)
-                })
-            } else { # fast integration over circular domains !
-                # effRange of each event
-                effRanges <- effRangeTypes[eventData$type]
-                # automatic choice of h (pixel spacing in image)
-                hs <- effRanges / nCub   # could e.g. equal sigma
-                # Compute the integral of 'siaf' over each influence region
-                siafInts <- numeric(length(.influenceRegion))
-                for(i in seq_along(siafInts)) {
-                    eps <- eventData$eps.s[i]
-                    bdisti <- .bdist[i]
-                    effRange <- effRanges[i]
-                    siafInts[i] <- if (effRange <= bdisti) {
-                            # effective region ("6 sigma") completely inside W
-                            siaf$Fcircle(min(eps,effRange), siafpars, as.integer(eventData[i,"type"]))
-                        } else if (eps <= bdisti) {
-                            # influence region is completely inside W
-                            siaf$Fcircle(eps, siafpars, as.integer(eventData[i,"type"]))
-                        } else {
-                            # integrate over polygonal influence region
-                            polyCub.midpoint(.influenceRegion[[i]], siaf$f,
-                                siafpars, as.integer(eventData[i,"type"]), eps = hs[i])
-                        }
-                }
-                siafInts
-            }
+        siafInts <- if (length(influenceRegion) == 0L) numeric(0L) else {
+            environment(.siafInt) <- environment()
+            .siafInt(siafpars)
+        }
         # Matrix of terms in the epidemic component
         eTerms <- cbind(
             qSum = qSumTypes[eventData$type],
             expeta = exp(drop(mme %*% gamma)),
-            siafInt = siafInt
+            siafInt = siafInts
         )
         # Return
-        list(eTerms, .bdist, .influenceRegion)
+        list(eTerms, bdist, influenceRegion)
     }
 
 
@@ -485,7 +477,7 @@ simEpidataCS <- function (endemic, epidemic, siaf, tiaf, qmatrix, rmarks,
         hIntWKt <- hIntWK[[as.character(tBLOCK)]]
 
         ## epidemic part
-        if (length(infectives) == 0L) ejIntWt <- numeric(0L) else {
+        ejIntWt <- if (!hase || length(infectives) == 0L) numeric(0L) else {
             eTerms <- eTerms[infectives,,drop=FALSE]
             gTerm <- if (upper) {
                     rep.int(gmax, length(infectives))
@@ -495,8 +487,7 @@ simEpidataCS <- function (endemic, epidemic, siaf, tiaf, qmatrix, rmarks,
                     tiaf$g(t-times, tiafpars, types)
                 }
             # ejIntWt only for infectives, others have 0
-            ejIntWt <- apply(cbind(eTerms,gTerm), 1, prod)
-            names(ejIntWt) <- infectives
+            structure(apply(cbind(eTerms,gTerm), 1, prod), names = infectives)
         }
 
         c("0"=hIntWKt, ejIntWt)   # endemic component has index "0" !
@@ -506,7 +497,7 @@ simEpidataCS <- function (endemic, epidemic, siaf, tiaf, qmatrix, rmarks,
     ### helper function calculating the integral of lambdag from oldct to ct
     ### during simulation; it depends on the current values of the simulation
 
-    add2Lambdag <- if (constanttiaf) {
+    add2Lambdag <- if (!hase || constanttiaf) {
             function () lambdagUpper * (ct-oldct)
         } else function () {
             # old endemic ground intensity * passed time
@@ -606,7 +597,7 @@ simEpidataCS <- function (endemic, epidemic, siaf, tiaf, qmatrix, rmarks,
         cat("\nSimulating (starting from t=", t0, ") ...\n", sep="")
     }
 
-    while(j <= maxEvents && ct < T)
+    while(j <= maxEvents && ct < T && (hash || length(infectives) > 0L))
     {
         loopCounter <- loopCounter + 1L
         if (trace > 0L && loopCounter %% trace == 0L) {
@@ -644,7 +635,7 @@ simEpidataCS <- function (endemic, epidemic, siaf, tiaf, qmatrix, rmarks,
         Delta <- tryCatch(rexp(1, rate = lambdagUpper),
             warning = function (w) {
                 if (lambdagUpper < 1) {   # rate was to small for rexp
-                    assign("hadNumericalProblems0", TRUE, inherits = TRUE)
+                    if (lambdagUpper > 0) assign("hadNumericalProblems0", TRUE, inherits = TRUE)
                     if (nextChangePoint == Inf) NULL else Inf
                 } else {   # rate was to big for rexp
                     0   # since R-2.7.0 rexp(1, Inf) returns 0 with no warning!
@@ -710,10 +701,20 @@ simEpidataCS <- function (endemic, epidemic, siaf, tiaf, qmatrix, rmarks,
                 .eventType <- sample(typeNames, 1L, prob=if (nbeta0 > 1L) exp(beta0))
                 stgrididx <- which(gridBlocks == tBLOCK)
                 .eventTile <- sample(stgrid$tile[stgrididx], 1L, prob=dsexpeta[stgrididx])  # this is a factor
-                # in spsample it is not guaranteed that the sample will consists of exactly n=1 point
-                while(inherits(
-                    eventLocationSP <- try(spsample(tiles[as.character(.eventTile),], n=1L, type="random"), silent = TRUE)
-                , "try-error")) {}  # if no point is sampled (very unlikely though) we would get an error
+                # spsample doesn't guarantee that the sample will consist of exactly n points
+                # if no point is sampled (very unlikely though), there would be an error
+                ntries <- 1L
+                while(
+                inherits(eventLocationSP <- try(
+                    spsample(tiles[as.character(.eventTile),],
+                             n=1L, type="random"),
+                    silent = TRUE), "try-error")) {
+                    if (ntries >= 1000) {
+                        stop("'spsample()' did not succeed in sampling a random point ",
+                             "in tile '", as.character(.eventTile), "'")
+                    }
+                    ntries <- ntries + 1L
+                }
                 .eventLocation <- coordinates(eventLocationSP)[1L,,drop=FALSE]
             } else {    # i.e. source is one of the currently infective individuals
                 sourceType <- eventMatrix[.eventSource,"type"]
@@ -727,9 +728,9 @@ simEpidataCS <- function (endemic, epidemic, siaf, tiaf, qmatrix, rmarks,
                         eventInsideIR <- FALSE
                         ntries <- 0L
                         while(!eventInsideIR) {
-                            if (ntries >= 1000) { # TODO: throw warning, skip to next event?
-                                cat("rejection sampling of event location seems difficult...\n")
-                                browser()
+                            if (ntries >= 1000) {
+                                stop("event location sampled by siaf$simulate() was ",
+                                     "rejected 1000 times (outside influence region)")
                             }
                             ntries <- ntries + 1L
                             eventLocationIR <- siaf$simulate(1L, siafpars, .eventTypeCode)
@@ -765,7 +766,7 @@ simEpidataCS <- function (endemic, epidemic, siaf, tiaf, qmatrix, rmarks,
             .offsetEvent <- attr(.mmhEvent, "offset")
             if (!is.null(.offsetEvent)) .etaEvent <- .offsetEvent + .etaEvent
             .lambdah <- exp(.etaEvent)
-            .lambdae <- if (length(.sources) > 0L) {
+            .lambdae <- if (hase && length(.sources) > 0L) {
                 .sdiffs <- .eventLocation[rep.int(1L,length(.sources)),,drop=FALSE] - eventCoords[.sources,,drop=FALSE]
                 .fSources <- siaf$f(.sdiffs, siafpars, eventMatrix[.sources,"type"])
                 .gSources <- tiaf$g(ct - eventMatrix[.sources,"time"], tiafpars, eventMatrix[.sources,"type"])
@@ -796,14 +797,23 @@ simEpidataCS <- function (endemic, epidemic, siaf, tiaf, qmatrix, rmarks,
     if (trace > 0L) cat("---\n")
 
     
-    ### update T if simulation ended because of limited 'nEvents'
+    ### update T if simulation ended preterm
 
-    if (j > maxEvents) {
+    if (j > maxEvents || (!hash && length(infectives) == 0L)) {
         T <- ct
         # clip stgrid to effective time range of simulation
         stgrid <- subset(stgrid, start <= T)
+        if (j > maxEvents) {
+            cat("Maximum number of events (nEvents=", nEvents,
+                ") reached @t = ", T, "\n", sep="")
+        } else { # epidemic-only model
+            cat("Simulation has ended preterm (no more infectives)",
+                "@t =", T, "with", j-1L-Nout, "simulated events.\n")
+        }
+    } else { # ct >= T or ct+Delta >= T
+        cat("Simulation has ended @t =", T, "with", j-1L-Nout,
+            "simulated events.\n")
     }
-    cat("Simulation has ended @t =", T, "with", j-1L-Nout, "simulated events.\n")
 
 
 
@@ -822,6 +832,13 @@ simEpidataCS <- function (endemic, epidemic, siaf, tiaf, qmatrix, rmarks,
     }
 
 
+    ### throw an error if no events have been simulated
+    
+    if (j-1L == Nout) {
+        stop("no events have been simulated")
+    }
+
+    
     ### transform eventMatrix back into a data.frame with original factor variables
 
     if (trace > 0L) {
@@ -865,10 +882,12 @@ simEpidataCS <- function (endemic, epidemic, siaf, tiaf, qmatrix, rmarks,
     ### Construct "epidataCS" object
 
     events <- SpatialPointsDataFrame(
-        coords = eventCoords[seqAlongEvents,,drop=FALSE],
-        data = eventData,
-        proj4string = CRS(proj4string(W)),
-        match.ID = FALSE
+        coords = eventCoords[seqAlongEvents,,drop=FALSE], data = eventData,
+        proj4string = CRS(proj4string(W)), match.ID = FALSE
+        #, bbox = bbox(W)   # the bbox of SpatialPoints is defined as the actual
+                            # bbox of the points and is also updated every time 
+                            # when subsetting the SpatialPoints object
+                            # -> useless to specify it as the bbox of W
     )
 
     if (.onlyEvents) {
@@ -893,9 +912,12 @@ simEpidataCS <- function (endemic, epidemic, siaf, tiaf, qmatrix, rmarks,
                    epidemic = formula(epidemic),
                    siaf = siaf, tiaf = tiaf
                    )
-    epi$coefficients <- list(beta0=beta0, beta=beta, gamma=gamma,
-                             siafpars=siafpars, tiafpars=tiafpars)
+    # coefficients as a numeric vector to be compatible with twinstim-methods
+    epi$coefficients <- coefs  #list(beta0=beta0, beta=beta, gamma=gamma,
+                               #     siafpars=siafpars, tiafpars=tiafpars)
     epi$npars <- c(nbeta0=nbeta0, p=p, q=q, nsiafpars=nsiafpars, ntiafpars=ntiafpars)
+    epi$nCub <- nCub                    # for R0.simEpidataCS
+    epi$nCub.adaptive <- nCub.adaptive  # for R0.simEpidataCS
     epi$call <- cl
     epi$runtime <- proc.time()[[3]] - ptm
     class(epi) <- c("simEpidataCS", "epidataCS", "list")
@@ -904,6 +926,15 @@ simEpidataCS <- function (endemic, epidemic, siaf, tiaf, qmatrix, rmarks,
 
 
 
+### R0-method for "simEpidataCS"-objects (wrapper for R0.twinstim)
+
+R0.simEpidataCS <- function (object, trimmed = TRUE, ...)
+{
+    R0.twinstim(object, newevents=object$events@data, trimmed = trimmed, ...)
+}
+
+
+
 
 
 
@@ -911,17 +942,20 @@ simEpidataCS <- function (endemic, epidemic, siaf, tiaf, qmatrix, rmarks,
 # A 'simulate' method for objects of class "twinstim".
 ################################################################################
 
-### TODO: actually stgrid's of simulations might have different time ranges when nEvents is active -> atm, simplify ignores this
+### FIXME: actually stgrid's of simulations might have different time ranges
+###        when nEvents is active -> atm, simplify ignores this
 
 simulate.twinstim <- function (object, nsim = 1, seed = NULL, data, tiles,
-    rmarks = NULL, t0 = NULL, T = NULL, nEvents = 1e5, nCub,
+    rmarks = NULL, t0 = NULL, T = NULL, nEvents = 1e5,
+    nCub = object$nCub, nCub.adaptive = object$nCub.adaptive,
     W = NULL, trace = FALSE, nCircle2Poly = 32, gmax = NULL, .allocate = 500, simplify = TRUE, ...)
 {
     ptm <- proc.time()[[3]]
     cl <- match.call()
 
 
-    ### Determine seed (copied from stats:::simulate.lm)
+    ### Determine seed (this part is copied from stats:::simulate.lm with
+    ### Copyright (C) 1995-2012 The R Core Team)
 
     if (!exists(".Random.seed", envir = .GlobalEnv, inherits = FALSE))
         runif(1)
@@ -950,8 +984,6 @@ simulate.twinstim <- function (object, nsim = 1, seed = NULL, data, tiles,
     epidemic <- formula(object)$epidemic
     # we don't need any reference to the original twinstim evaluation environment
     environment(endemic) <- environment(epidemic) <- globalenv()
-    siaf     <- formula(object)$siaf
-    tiaf     <- formula(object)$tiaf
     if (is.null(rmarks)) {
         observedMarks <- subset(marks(data), subset = time > t0 & time <= T)
         observedMarks <- observedMarks[match("eps.t", names(observedMarks)):(ncol(observedMarks)-2L)]
@@ -972,16 +1004,27 @@ simulate.twinstim <- function (object, nsim = 1, seed = NULL, data, tiles,
 
     ### Run the simulation(s)
 
+    # establish call
+    simcall <- call("simEpidataCS", endemic=endemic, epidemic=epidemic,
+                    siaf=quote(formula(object)$siaf),
+                    tiaf=quote(formula(object)$tiaf),
+                    qmatrix=quote(object$qmatrix),
+                    rmarks=quote(rmarks), events=quote(data$events),
+                    stgrid=quote(data$stgrid), tiles=quote(tiles), beta0=beta0,
+                    beta=beta, gamma=gamma, siafpars=siafpars,
+                    tiafpars=tiafpars, t0=t0, T=T, nEvents=nEvents,
+                    nCub=nCub, nCub.adaptive=nCub.adaptive,
+                    W=quote(W), trace=trace, nCircle2Poly=nCircle2Poly,
+                    gmax=gmax, .allocate=.allocate,
+                    .skipChecks=TRUE, .onlyEvents=FALSE)
+    
     # First simulation
     if (nsim > 1L) {
         cat("\nTime at beginning of simulation:", as.character(Sys.time()), "\n")
         cat("Simulation 1 /", nsim, "...\n")
         cat("-------------------------------------------------------------------------------\n")
     }
-    res <- simEpidataCS(endemic, epidemic, siaf, tiaf, object$qmatrix, rmarks,
-                        data$events, data$stgrid, tiles, beta0, beta, gamma, siafpars,
-                        tiafpars, t0, T, nEvents, nCub, W, trace, nCircle2Poly,
-                        gmax, .allocate, .skipChecks = TRUE)
+    res <- eval(simcall)
     if (nsim > 1L) {
         cat("\n-------------------------------------------------------------------------------\n")
         cat("Runtime of first simulation:", res$runtime, "seconds\n")
@@ -1000,16 +1043,16 @@ simulate.twinstim <- function (object, nsim = 1, seed = NULL, data, tiles,
         # force garbage collection
         capture.output(gc())
         # run the remaining simulations
+        simcall$.onlyEvents <- simplify
         for (i in 2:nsim) {
             cat("Simulation", sprintf(paste0("%",nchar(nsim),"i"), i), "/", nsim, "...")
-            capture.output(
-            resi <- simEpidataCS(endemic, epidemic, siaf, tiaf, object$qmatrix, rmarks,
-                data$events, data$stgrid, tiles, beta0, beta, gamma, siafpars,
-                tiafpars, t0, T, nEvents, nCub, W, trace, nCircle2Poly,
-                gmax, .allocate, .skipChecks = TRUE, .onlyEvents = simplify)
-            )
-            cat("\tsimulated", if (simplify) sum(!is.na(resi$source)) else sum(!is.na(resi$events$source)),
-                "events up to time", if (simplify) attr(resi,"timeRange")[2] else resi$timeRange[2], "\n")
+            capture.output(resi <- eval(simcall))
+            .nEvents <- if (simplify) sum(!is.na(resi$source)) else {
+                sum(!is.na(resi$events$source))
+            }
+            .T <- if (simplify) attr(resi,"timeRange")[2] else resi$timeRange[2]
+            cat("\tsimulated", .nEvents, "events", if (nEvents == .nEvents)
+                "(reached maximum)", "up to time", .T, "\n")
             if (simplify) res$eventsList[[i]] <- resi else res[[i]] <- resi
         }
         cat("\nDone (", as.character(Sys.time()), ").\n", sep="")
@@ -1039,3 +1082,36 @@ print.simEpidataCSlist <- function (x, ...)
         "simulated epidemics of class \"simEpidataCS\" (not printed)\n\n")
     invisible(x)
 }
+
+"[[.simEpidataCSlist" <- function (x, i) {
+    simplified <- attr(x, "simplified")
+    if (simplified) {
+        x <- unclass(x)
+        x$eventsList <- x$eventsList[[i]]
+        names(x)[names(x) == "eventsList"] <- "events"
+        x <- append(x, list(timeRange = attr(x$events, "timeRange")), after=4L)
+        x$runtime <- attr(x$events, "runtime")
+        attr(x$events, "timeRange") <- attr(x$events, "runtime") <- NULL
+        class(x) <- c("simEpidataCS", "epidataCS", "list")
+        x
+    } else NextMethod("[[")
+}
+
+plot.simEpidataCSlist <- function (x,
+    which = NULL, mfrow = n2mfrow(length(which)),
+    main = paste("Simulated epidemic", which),
+    aggregate = c("time", "space"), subset, ...)
+{
+    simplified <- attr(x, "simplified")
+    nsim <- if (simplified) length(x$eventsList) else length(x)
+    if (is.null(which)) {
+        which <- seq_len(nsim)
+        if (nsim > 4) which <- sample(which, 4L)
+    }
+    opar <- par(mfrow = mfrow); on.exit(par(opar))
+    main <- rep(main, length.out=length(which))
+    for (i in which) {
+        do.call("plot", args=list(x=quote(x[[i]]), aggregate=aggregate,
+                        subset=substitute(subset), main = main[i], ...))
+    }
+}
